from pbr.version import VersionInfo

_v = VersionInfo('scopus').semantic_version()
__version__ = _v.release_string()
version_info = _v.version_tuple()

from scopus.utils import *

from scopus.abstract_citations import *
from scopus.scopus_affiliation import *
from scopus.scopus_affiliation import *
from scopus.scopus_author import *
from scopus.scopus_reports import *
from scopus.scopus_search import *
from scopus.search_author import *



import os
import sys
import textwrap
import time
import xml.etree.ElementTree as ET
from collections import Counter, namedtuple
from operator import itemgetter

from .scopus_api import ScopusAbstract
from .scopus_search import ScopusSearch
from .scopus_affiliation import ScopusAffiliation
from scopus.utils import download, get_content, get_encoded_text, ns

SCOPUS_AUTHOR_DIR = os.path.expanduser('~/.scopus/author')

if not os.path.exists(SCOPUS_AUTHOR_DIR):
    os.makedirs(SCOPUS_AUTHOR_DIR)


class ScopusAuthor(object):
    @property
    def author_id(self):
        """The scopus id for the author."""
        return self._author_id

    @property
    def orcid(self):
        """The author's ORCID."""
        return self._orcid

    @property
    def hindex(self):
        """The author hindex"""
        return self._hindex

    @property
    def ndocuments(self):
        """Number of documents authored (excludes book chapters and notes)."""
        return self._ndocuments

    @property
    def ncited_by(self):
        """Total number of citing authors."""
        return self._ncited_by

    @property
    def citation_count(self):
        """Total number of citing items."""
        return self._citation_count

    @property
    def ncoauthors(self):
        """Total number of coauthors."""
        return self._ncoauthors

    @property
    def current_affiliation(self):
        """Current affiliation according to scopus."""
        return self._current_affiliation

    @property
    def affiliation_history(self):
        """List of ScopusAffiliation objects."""
        return self._affiliation_history

    @property
    def date_created(self):
        """Date the Scopus record was created."""
        return self._date_created

    @property
    def firstname(self):
        """Author first name."""
        return self._firstname

    @property
    def lastname(self):
        """Author last name."""
        return self._lastname

    @property
    def name(self):
        """Author name."""
        return self._name

    @property
    def scopus_url(self):
        """URL to the author's profile page."""
        return self._scopus_url

    @property
    def citedby_url(self):
        """URL to Scopus page of citing papers."""
        return self._citedby_url

    @property
    def coauthor_url(self):
        """URL to Scopus coauthor page."""
        return self._coauthor_url

    @property
    def subject_areas(self):
        """List of tuples of author subject areas in the form
        (area, frequency, abbreviation, code), where frequency is the
        number of publications in this subject area.
        """
        freqs = self._subject_freq
        return [(a.text, freqs[int(a.get("code"))], a.get("abbrev"), a.get("code"))
                for a in self._area_elements]

    @property
    def publication_history(self):
        """List of tuples of authored publications in the form
        (title, abbreviation, type, issn), where issn is only given
        for journals.  abbreviation and issn may be None.
        """
        hist = []
        for pub in self._pub_hist:
            try:
                issn = pub.find("issn").text
            except AttributeError:
                issn = None
            try:
                abbr = pub.find("sourcetitle-abbrev").text
            except AttributeError:
                abbr = None
            hist.append((pub.find("sourcetitle").text, abbr, pub.get("type"), issn))
        return hist

    def __init__(self, author_id, refresh=False, refresh_aff=False, level=1):
        """Class to represent a Scopus Author query by the scopus-id.

        Parameters
        ----------
        author_id : str or int
            The ID of the author to search for.  Optionally expressed
            as an Elsevier EID (i.e., in the form 9-s2.0-nnnnnnnn).

        refresh : bool (optional, default=False)
            Whether to refresh the cached file (if it exists) or not.

        refresh_aff : bool (optional, default=False)
            Whether to refresh the cached corresponding affiliation views
            (if they exist) or not.

        level : int (optional, default=1)
            Number of * to print in property __str__.

        Notes
        -----
        The files are cached in ~/.scopus/author/{author_id} (without
        eventually leading '9-s2.0-').
        """
        author_id = str(int(str(author_id).split('-')[-1]))
        self.level = level

        qfile = os.path.join(SCOPUS_AUTHOR_DIR, author_id)
        url = ('https://api.elsevier.com/content/author/'
               'author_id/{}').format(author_id)
        params = {'author_id': author_id, 'view': 'ENHANCED'}
        xml = ET.fromstring(get_content(qfile, url=url, refresh=refresh,
                                        params=params))
        self.xml = xml
        self._orcid = get_encoded_text(xml, 'coredata/orcid')
        hindex = get_encoded_text(xml, 'h-index')
        self._hindex = int(hindex) if hindex is not None else 0

        ndocuments = get_encoded_text(xml, 'coredata/document-count')
        self._ndocuments = int(ndocuments) if ndocuments is not None else 0

        _author_id = get_encoded_text(xml, 'coredata/dc:identifier')
        self._author_id = _author_id.split(":")[-1]

        citation_count = get_encoded_text(xml, 'coredata/citation-count')
        self._citation_count = int(citation_count) if citation_count is not None else 0

        ncited_by = get_encoded_text(xml, 'coredata/cited-by-count')
        self._ncited_by = int(ncited_by) if ncited_by is not None else 0

        ncoauthors = get_encoded_text(xml, 'coauthor-count')
        self._ncoauthors = int(ncoauthors) if ncoauthors is not None else 0

        self._current_affiliation = get_encoded_text(xml,
            'author-profile/affiliation-current/affiliation/ip-doc/afdispname')

        # affiliation history (sort out faulty historic affiliations)
        aff_ids = [el.attrib.get('affiliation-id') for el in
                   xml.findall('author-profile/affiliation-history/affiliation')
                   if el is not None and len(list(el.find("ip-doc").iter())) > 1]
        affs = [ScopusAffiliation(aff_id, refresh=refresh_aff) for aff_id in aff_ids]
        self._affiliation_history = affs

        date_created = xml.find('author-profile/date-created', ns)
        if date_created is not None:
            self._date_created = (int(date_created.attrib['year']),
                                  int(date_created.attrib['month']),
                                  int(date_created.attrib['day']))
        else:
            self._date_created = (None, None, None)
        # Research areas
        self._area_elements = xml.findall('subject-areas/subject-area')
        # {code: name}
        d = {int(ae.attrib['code']): ae.text for ae in self._area_elements}

        freqs = xml.findall('author-profile/classificationgroup/'
                            'classifications[@type="ASJC"]/classification')
        # {code: frequency}
        c = {int(cls.text): int(cls.attrib['frequency'])
             for cls in freqs}
        self._subject_freq = c

        categories = [(d[code], c[code]) for code in d]
        categories.sort(reverse=True, key=itemgetter(1))
        self.categories = categories

        self._firstname = (get_encoded_text(xml,
            'author-profile/preferred-name/given-name') or '')

        self._lastname = (get_encoded_text(xml,
           'author-profile/preferred-name/surname') or '')

        self._name = ((get_encoded_text(xml,
                       'author-profile/preferred-name/given-name') or '') +
                      ' ' +
                      (get_encoded_text(xml,
                       'author-profile/preferred-name/surname') or ''))

        # Real website for the author
        self._scopus_url = xml.find('coredata/link[@rel="scopus-author"]')
        if self._scopus_url is not None:
            self._scopus_url = self._scopus_url.get('href')

        # API URL for coauthors
        self._coauthor_url = xml.find('coredata/link[@rel="coauthor-search"]')
        if self._coauthor_url is not None:
            self._coauthor_url = self._coauthor_url.get('href')

        # Publication history
        pub_hist_elements = self.xml.findall('author-profile/journal-history/')
        self._pub_hist = pub_hist_elements

    def get_coauthors(self):
        """Return list of coauthors, their scopus-id and research areas."""
        url = self.xml.find('coredata/link[@rel="coauthor-search"]').get('href')
        xml = download(url=url).text.encode('utf-8')
        xml = ET.fromstring(xml)
        coauthors = []
        N = int(get_encoded_text(xml, 'opensearch:totalResults') or 0)

        AUTHOR = namedtuple('Author',
                            ['name', 'scopus_id', 'affiliation', 'categories'])

        count = 0
        while count < N:
            params = {'start': count, 'count': 25}
            xml = download(url=url, params=params).text.encode('utf-8')
            xml = ET.fromstring(xml)

            for entry in xml.findall('atom:entry', ns):

                given_name = get_encoded_text(entry,
                    'atom:preferred-name/atom:given-name')
                surname = get_encoded_text(entry,
                    'atom:preferred-name/atom:surname')
                coauthor_name = u'{0} {1}'.format(given_name, surname)

                scopus_id = get_encoded_text(entry,
                    'dc:identifier').replace('AUTHOR_ID:', '')

                affiliation = get_encoded_text(entry,
                    'atom:affiliation-current/atom:affiliation-name')

                # get categories for this author
                s = u', '.join(['{0} ({1})'.format(subject.text,
                                                  subject.attrib['frequency'])
                               for subject in
                               entry.findall('atom:subject-area', ns)])

                coauthors += [AUTHOR(coauthor_name, scopus_id, affiliation, s)]
            count += 25

        return coauthors

    def get_document_eids(self, *args, **kwds):
        """Return list of EIDs for the author using ScopusSearch."""
        search = ScopusSearch('au-id({})'.format(self.author_id),
                              *args, **kwds)
        return search.EIDS

    def get_abstracts(self, refresh=True):
        """Return a list of ScopusAbstract objects using ScopusSearch."""
        return [ScopusAbstract(eid, refresh=refresh)
                for eid in self.get_document_eids(refresh=refresh)]

    def get_document_summary(self, N=None, cite_sort=True, refresh=True):
        """Return a summary string of documents.

        Parameters
        ----------
        N : int or None (optional, default=None)
            Maximum number of documents to include in the summary.
            If None, return all documents.

        cite_sort : bool (optional, default=True)
            Whether to sort xml by number of citations, in decreasing order,
            or not.

        refresh : bool (optional, default=True)
            Whether to refresh the cached abstract file (if it exists) or not.

        Returns
        -------
        s : str
            Text summarizing an author's documents.
        """
        abstracts = [ScopusAbstract(eid, refresh=refresh)
                     for eid in self.get_document_eids(refresh=refresh)]

        if cite_sort:
            counts = [(a, int(a.citedby_count)) for a in abstracts]
            counts.sort(reverse=True, key=itemgetter(1))
            abstracts = [a[0] for a in counts]

        if N is None:
            N = len(abstracts)

        s = [u'{0} of {1} documents'.format(N, len(abstracts))]

        for i in range(N):
            s += ['{0:2d}. {1}\n'.format(i + 1, str(abstracts[i]))]

        return '\n'.join(s)

    def __str__(self):
        """Return a summary string."""
        s = ['{} {} (updated on {})'.format(
             '*' * self.level, self._name, time.asctime())]

        url = self.xml.find('coredata/link[@rel="scopus-author"]')
        if url is not None:
            url = url.get('href', 'None')
        else:
            url = ''

        s += ['']

        orcid = get_encoded_text(self.xml, 'coredata/orcid')
        if orcid is not None:
            s += ['http://orcid.org/' + orcid]

        s += ['{} documents cited {} times by {} people ({} coauthors)'.format(
              self._ndocuments, self._citation_count, self._ncited_by,
              self._ncoauthors)]
        s += ['#first author papers {0}'.format(self.n_first_author_papers())]
        s += ['#last author papers {0}'.format(self.n_last_author_papers())]
        s += ['h-index: {}'.format(self._hindex) +
              '        AIF(2014) = ' +
              '{0:1.2f}'.format(self.author_impact_factor(2015)[2])]

        s += ['Scopus ID created on {}'.format(self.date_created)]

        # Current Affiliation. Note this is what Scopus thinks is current.
        s += ['\nCurrent affiliation according to Scopus:']
        s += ['  ' + (self._current_affiliation or '')]

        # subject areas
        s += ['\nSubject areas']

        s += [textwrap.fill(', '.join(['{0} ({1})'.format(el[0], el[1])
                                       for el in self.categories]),
                            initial_indent='  ',
                            subsequent_indent='  ')]

        # journals published in
        temp_s = [el.find('sourcetitle-abbrev').text for el in self._pub_hist]
        s += ['\nPublishes in:\n' +
              textwrap.fill(', '.join(temp_s), initial_indent='  ',
                            subsequent_indent='  ')]

        # affiliation history
        s += ['\nAffiliation history:']
        for aff in self.affiliation_history:
            s += [str(aff)]

        # print a bibliography
        s += [self.get_document_summary()]

        return '\n'.join(s)

    def author_impact_factor(self, year=2014, refresh=True):
        """Get author_impact_factor for the .

        Parameters
        ----------
        year : int (optional, default=2014)
            The year based for which the impact factor is to be calculated.

        refresh : bool (optional, default=True)
            Whether to refresh the cached search file (if it exists) or not.

        Returns
        -------
        (ncites, npapers, aif) : tuple of integers
            The citations count, publication count, and author impact factor.
        """
        scopus_abstracts = [ScopusAbstract(eid, refresh=refresh)
                            for eid in self.get_document_eids(refresh=refresh)
                            if ScopusAbstract(eid, refresh=refresh).aggregationType == 'Journal']

        cites = [int(ab.citedby_count) for ab in scopus_abstracts]
        years = [int(ab.coverDate.split('-')[0]) for ab in scopus_abstracts]

        data = zip(years, cites, scopus_abstracts)
        data = sorted(data, key=itemgetter(1), reverse=True)

        # now get aif papers for year-1 and year-2
        aif_data = [tup for tup in data if tup[0] in (year - 1, year - 2)]
        Ncites = sum([tup[1] for tup in aif_data])
        if len(aif_data) > 0:
            return (Ncites, len(aif_data), Ncites / float(len(aif_data)))
        else:
            return (Ncites, len(aif_data), 0)

    def n_first_author_papers(self, refresh=True):
        """Return number of papers with author as the first author."""
        scopus_abstracts = [ScopusAbstract(eid, refresh=refresh)
                            for eid in self.get_document_eids(refresh=refresh)
                            if ScopusAbstract(eid, refresh=refresh).aggregationType == 'Journal']
        first_authors = [1 for ab in scopus_abstracts
                         if ab.authors[0].scopusid == self.author_id]

        return sum(first_authors)

    def n_last_author_papers(self, refresh=True):
        """Return number of papers with author as the last author."""
        scopus_abstracts = [ScopusAbstract(eid, refresh=refresh)
                            for eid in self.get_document_eids(refresh=refresh)
                            if ScopusAbstract(eid, refresh=refresh).aggregationType == 'Journal']
        first_authors = [1 for ab in scopus_abstracts
                         if ab.authors[-1].scopusid == self.author_id]
        return sum(first_authors)

    def n_journal_articles(self, refresh=True):
        """Return the number of journal articles."""
        return len([ScopusAbstract(eid, refresh=refresh)
                    for eid in self.get_document_eids(refresh=refresh)
                    if ScopusAbstract(eid, refresh=refresh).aggregationType == 'Journal'])

    def n_yearly_publications(self, refresh=True):
        """Number of journal publications in a given year."""
        scopus_abstracts = [ScopusAbstract(eid, refresh=refresh)
                            for eid in self.get_document_eids(refresh=refresh)
                            if ScopusAbstract(eid, refresh=refresh).aggregationType == 'Journal']
        pub_years = [int(ab.coverDate.split('-')[0]) for ab in scopus_abstracts]
        return Counter(pub_years)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for `CitationOverview` module."""

from collections import namedtuple

from nose.tools import assert_equal, assert_true

import scopus


co = scopus.CitationOverview("2-s2.0-84930616647", refresh=True,
                             start=2015, end=2017)


def test_authors():
    Author = namedtuple('Author', 'name surname initials id url')
    url = 'https://api.elsevier.com/content/author/author_id/7004212771'
    expected = [Author(name='Kitchin J.R.', surname='Kitchin',
                initials='J.R.', id='7004212771',
                url=url)]
    assert_equal(co.authors, expected)


def test_cc():
    assert_equal(co.cc, [(2015, '0'), (2016, '4'), (2017, '2')])


def test_citationType_long():
    assert_equal(co.citationType_long, 'Review')


def test_citationType_short():
    assert_equal(co.citationType_short, 're')


def test_doi():
    assert_equal(co.doi, '10.1021/acscatal.5b00538')


def test_endingPage():
    assert_equal(co.endingPage, '3899')


def test_h_index():
    assert_equal(co.h_index, '1')


def test_issn():
    assert_equal(co.issn, '2155-5435')


def test_issueIdentifier():
    assert_equal(co.issueIdentifier, '6')


def test_lcc():
    assert_equal(co.lcc, '0')


def test_pcc():
    assert_equal(co.pcc, '0')


def test_pii():
    assert_equal(co.pii, None)


def test_publicationName():
    assert_equal(co.publicationName, 'ACS Catalysis')


def test_rangeCount():
    assert_equal(co.rangeCount, '6')


def test_rowTotal():
    assert_equal(co.rowTotal, '6')


def test_scopus_id():
    assert_equal(co.scopus_id, '84930616647')


def test_startingPage():
    assert_equal(co.startingPage, '3894')


def test_title():
    expected = 'Examples of effective data sharing in scientific publishing'
    assert_equal(co.title, expected)


def test_url():
    expected = 'https://api.elsevier.com/content/abstract/scopus_id/84930616647'
    assert_equal(co.url, expected)


def test_volume():
    assert_equal(co.volume, '5')



import os
from collections import namedtuple
from datetime import datetime
from json import loads

from scopus.utils import get_content

CITATION_OVERVIEW_DIR = os.path.expanduser('~/.scopus/citation_overview')

if not os.path.exists(CITATION_OVERVIEW_DIR):
    os.makedirs(CITATION_OVERVIEW_DIR)


class CitationOverview(object):

    @property
    def authors(self):
        """A list of namedtuples storing author information,
        where each namedtuple corresponds to one author.
        The information in each namedtuple is (name surname initials id url).
        All entries are strings.
        """
        out = []
        order = 'name surname initials id url'
        auth = namedtuple('Author', order)
        for author in self.citeInfoMatrix.get('author'):
            author = {k.split(":", 1)[-1]: v for k, v in author.items()}
            new = auth(name=author.get('index-name'),
                       surname=author.get('surname'),
                       initials=author.get('initials'),
                       id=author.get('authid'),
                       url=author.get('author-url'))
            out.append(new)
        return out

    @property
    def cc(self):
        """List of tuples of yearly number of citations
        for specified years."""
        _years = range(self.start, self.end+1)
        try:
            return list(zip(_years, [d.get('$') for d in self.citeInfoMatrix['cc']]))
        except AttributeError:  # No citations
            return list(zip(_years, [0]*len(_years)))

    @property
    def citationType_long(self):
        """Type (long version) of the abstract (e.g. article, review)."""
        return self.citeInfoMatrix.get('citationType', {}).get('$')

    @property
    def citationType_short(self):
        """Type (short version) of the abstract (e.g. ar, re)."""
        return self.citeInfoMatrix.get('citationType', {}).get('@code')

    @property
    def doi(self):
        """Document Object Identifier (DOI) of the abstract."""
        return self.identifierlegend.get('doi')

    @property
    def endingPage(self):
        """Ending page."""
        return self.citeInfoMatrix.get('endingPage')

    @property
    def h_index(self):
        """h-index of ciations of the abstract (according to Scopus)."""
        return self.hindex

    @property
    def issn(self):
        """ISSN of the publisher.
        Note: If E-ISSN is known to Scopus, this returns both
        ISSN and E-ISSN in random order separated by blank space.
        """
        return self.citeInfoMatrix.get('issn')

    @property
    def issueIdentifier(self):
        """Issue number for abstract."""
        return self.citeInfoMatrix.get('issueIdentifier')

    @property
    def lcc(self):
        """Number of citations the abstract received
        after the specified end year.
        """
        return self.citeInfoMatrix.get('lcc')

    @property
    def pcc(self):
        """Number of citations the abstract received
        before the specified start year.
        """
        return self.citeInfoMatrix.get('pcc')

    @property
    def pii(self):
        """The Publication Item Identifier (PII) of the abstract."""
        return self.identifierlegend.get('pii')

    @property
    def publicationName(self):
        """Name of source the abstract is published in (e.g. the Journal)."""
        return self.citeInfoMatrix.get('publicationName')

    @property
    def scopus_id(self):
        """The Scopus ID of the abstract.  It is the second part of an EID.
        The Scopus ID might differ from the one provided."""
        return self.identifierlegend.get('scopus_id')

    @property
    def startingPage(self):
        """Starting page."""
        return self.citeInfoMatrix.get('startingPage')

    @property
    def rangeCount(self):
        """Number of citations for specified years."""
        return self.citeInfoMatrix.get('rangeCount')

    @property
    def rowTotal(self):
        """Number of citations (specified and omitted years)."""
        return self.citeInfoMatrix.get('rowTotal')

    @property
    def title(self):
        """Abstract title."""
        return self.citeInfoMatrix.get('title')

    @property
    def url(self):
        """URL to Citation Overview API view of the abstract."""
        return self.citeInfoMatrix.get('url')

    @property
    def volume(self):
        """Volume for the abstract."""
        return self.citeInfoMatrix.get('volume')

    def __init__(self, eid, start, end=datetime.now().year, refresh=False):
        """Class to represent the results from a Scopus Citation Overview.
        See https://api.elsevier.com/documentation/guides/AbstractCitationViews.htm.

        Parameters
        ----------
        eid : str
            The EID of the abstract.

        start : str or int
            The first year for which the citation count should be loaded

        end : str or int (optional, default=datetime.now().year)
            The last year for which the citation count should be loaded.
            Default is the current year.

        refresh : bool (optional, default=False)
            Whether to refresh the cached file if it exists or not.

        Notes
        -----
        The files are cached in ~/.scopus/citation_overview/{eid}.
        Your API Key needs to be approved by Elsevier to access this view.
        """
        # Get file content
        scopus_id = eid.split('0-')[-1]
        qfile = os.path.join(CITATION_OVERVIEW_DIR, eid)
        url = "https://api.elsevier.com/content/abstract/citations/{}".format(scopus_id)
        params = {'scopus_id': scopus_id, 'date': '{}-{}'.format(start, end)}
        res = get_content(qfile, url=url, refresh=refresh, params=params,
                          accept='json')
        data = loads(res.decode('utf-8'))['abstract-citations-response']

        self.start = int(start)
        self.end = int(end)

        # citeInfoMatrix
        m = data['citeInfoMatrix']['citeInfoMatrixXML']['citationMatrix']['citeInfo'][0]
        self.citeInfoMatrix = {k.split(":", 1)[-1]: v for k, v in m.items()}
        # h-index
        self.hindex = data['h-index']
        # identifier-legend
        l = data['identifier-legend']['identifier'][0]
        self.identifierlegend = {k.split(":", 1)[-1]: v for k, v in l.items()}
        # citeColumnTotalXML
        self.citeColumnTotalXML = data['citeColumnTotalXML']  # not used



import hashlib
import os
import sys
from collections import namedtuple
from json import dumps, loads

from scopus.utils import download, get_content

AUTHOR_SEARCH_DIR = os.path.expanduser('~/.scopus/author_search')

if not os.path.exists(AUTHOR_SEARCH_DIR):
    os.makedirs(AUTHOR_SEARCH_DIR)


class AuthorSearch(object):
    @property
    def authors(self):
        """A list of namedtuples storing author information,
        where each namedtuple corresponds to one author.
        The information in each namedtuple is (eid surname initials givenname
        documents affiliation affiliation_id city country areas).

        All entries are strings or None.  Areas combines abbreviated subject
        areas followed by the number of documents in this subject.
        """
        out = []
        order = 'eid surname initials givenname affiliation documents '\
                'affiliation_id city country areas'
        auth = namedtuple('Author', order)
        for item in self._json:
            name = item.get('preferred-name', {})
            aff = item.get('affiliation-current', {})
            fields = item.get('subject-area',
                              [{'@abbrev': '', '@frequency': ''}])
            areas = ["{} ({})".format(d.get('@abbrev', ''), d.get('@frequency', ''))
                     for d in fields]
            new = auth(eid=item['eid'],
                       surname=name.get('surname'),
                       initials=name.get('initials'),
                       givenname=name.get('given-name'),
                       documents=item.get('document-count', '0'),
                       affiliation=aff.get('affiliation-name'),
                       affiliation_id=aff.get('affiliation-id'),
                       city=aff.get('affiliation-city'),
                       country=aff.get('affiliation-country'),
                       areas="; ".join(areas))
            out.append(new)
        return out

    def __init__(self, query, count=200, start=0,
                 max_entries=5000, refresh=False):
        """Class to search a query, and retrieve a list of author IDs as results.

        Parameters
        ----------
        query : str
            A string of the query, e.g. "authlast(Einstein) and
            authfirst(Albert)".

        count : int (optional, default=200)
            The number of entries to be displayed at once.  A smaller number
            means more queries with each query having less results.

        start : int (optional, default=0)
            The entry number of the first search item to start with.

        refresh : bool (optional, default=False)
            Whether to refresh the cached file if it exists or not.

        max_entries : int (optional, default=5000)
            Raise error when the number of results is beyond this number.
            The Scopus Search Engine does not allow more than 5000 entries.

        Raises
        ------
        Exception
            If the number of search results exceeds max_entries.

        Notes
        -----
        Json results are cached in ~/.scopus/author_search/{fname}, where
        fname is the hashed version of query.

        The results are stored as a property named authors.
        """

        self.query = query
        qfile = os.path.join(AUTHOR_SEARCH_DIR,
                             hashlib.md5(query.encode('utf8')).hexdigest())

        if os.path.exists(qfile) and not refresh:
            self._json = []
            with open(qfile) as f:
                for r in f.readlines():
                    self._json.append(loads(r))
        else:
            # No cached file exists, or we are refreshing.
            # First, we get a count of how many things to retrieve
            url = 'https://api.elsevier.com/content/search/author'
            params = {'query': query, 'count': 0, 'start': 0}
            res = get_content(qfile, url=url, refresh=refresh, params=params,
                              accept='json')
            data = loads(res.decode('utf-8'))['search-results']

            N = int(data.get('opensearch:totalResults', 0))

            if N > max_entries:
                raise Exception(('N = {}. '
                                 'Set max_entries to a higher number or '
                                 'change your query ({})').format(N, query))

            self._json = []
            while N > 0:
                params = {'query': query, 'count': count, 'start': start}
                resp = download(url=url, params=params, accept="json")
                results = resp.json()

                if 'entry' in results.get('search-results', []):
                    for r in results['search-results']['entry']:
                        self._json.append({f: r[f] for f in r.keys()})
                start += count
                N -= count

            with open(qfile, 'wb') as f:
                for author in self._json:
                    f.write('{}\n'.format(dumps(author)).encode('utf-8'))

    def __str__(self):
        s = """{query}
        Resulted in {N} hits.
    {entries}"""
        return s.format(query=self.query, N=len(self._json),
                        entries='\n    '.join([str(a) for a in self._json]))



# Namespaces for Scopus XML
ns = {'dtd': 'http://www.elsevier.com/xml/svapi/abstract/dtd',
      'dn': 'http://www.elsevier.com/xml/svapi/abstract/dtd',
      'ait': "http://www.elsevier.com/xml/ani/ait",
      'cto': "http://www.elsevier.com/xml/cto/dtd",
      'xocs': "http://www.elsevier.com/xml/xocs/dtd",
      'ce': 'http://www.elsevier.com/xml/ani/common',
      'prism': 'http://prismstandard.org/namespaces/basic/2.0/',
      'xsi': "http://www.w3.org/2001/XMLSchema-instance",
      'dc': 'http://purl.org/dc/elements/1.1/',
      'atom': 'http://www.w3.org/2005/Atom',
      'opensearch': 'http://a9.com/-/spec/opensearch/1.1/'}


def get_encoded_text(container, xpath):
    """Return text for element at xpath in the container xml if it is there.

    Parameters
    ----------
    container : xml.etree.ElementTree.Element
        The element to be searched in.

    xpath : str
        The path to be looked for.

    Returns
    -------
    result : str
    """
    try:
        return container.find(xpath, ns).text
    except AttributeError:
        return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for `ScopusAuthor` module."""

from collections import Counter
from nose.tools import assert_equal, assert_true

import scopus


au = scopus.ScopusAuthor("7004212771", refresh=True)


def test_affiliation_history():
    affs = au.affiliation_history
    assert_true(len(affs) >= 16)
    assert_true(isinstance(affs[0], scopus.scopus_affiliation.ScopusAffiliation))


def test_author_id():
    assert_equal(au.author_id, "7004212771")


def test_author_impact_factor():
    (ncites, npapers, aif) = au.author_impact_factor(refresh=False)
    expected = (504, 11, 45.81818181818182)
    assert_true(expected[0] <= ncites)
    assert_true(expected[1] <= npapers)
    assert_true(expected[2] <= aif)


def test_citation_count():
    expected = 5514
    assert_true(au.citation_count >= expected)


def test_coauthor_url():
    expected = 'http://api.elsevier.com/content/search/author?co-author=7004212771'
    assert_equal(au.coauthor_url, expected)


def test_current_affiliation():
    expected = 'Carnegie Mellon University, Department of Chemical Engineering'
    assert_equal(au.current_affiliation, expected)


def test_date_created():
    assert_equal(au.date_created, (2005, 12, 3))


def test_firstname():
    assert_equal(au.firstname, 'John R.')


def test_get_abstracts():
    abstracts = au.get_abstracts(refresh=False)
    assert_true(len(abstracts) >= 90)
    assert_true(isinstance(abstracts[0], scopus.scopus_api.ScopusAbstract))


def test_get_coauthors():
    coauthors = au.get_coauthors()
    assert_true(len(coauthors) >= 158)


def test_get_document_summary():
    pass


def n_last_author_papers():
    expected = 44
    assert_true(au.n_last_author_papers(refresh=False) >= 44)


def test_hindex():
    expected = 22
    assert_true(au.hindex >= expected)


def test_lastname():
    assert_equal(au.lastname, 'Kitchin')


def test_name():
    assert_equal(au.name, 'John R. Kitchin')


def test_ncited_by():
    expected = 4335
    assert_true(au.ncited_by >= expected)


def test_ncoauthors():
    expected = 153
    assert_true(au.ncoauthors >= expected)


def test_ndocuments():
    expected = 90
    assert_true(au.ndocuments >= expected)


def test_n_first_author_papers():
    expected = 12
    assert_true(au.n_first_author_papers(refresh=False) >= 12)


def test_n_journal_articles():
    jour_arts = au.n_journal_articles(refresh=False)
    expected = 72
    assert_true(jour_arts >= 72)


def test_n_yearly_publications():
    yearly_pubs = au.n_yearly_publications(refresh=False)
    assert_true(isinstance(yearly_pubs, Counter))


def test_orcid():
    assert_equal(au.orcid, '0000-0003-2625-9232')


def publication_history():
    hist = au.publication_history
    assert_true(isinstance(areas, list))
    assert_true(len(areas) > 0)
    expected = ('ACS Catalysis', 'ACS Catal.', 'j', '21555435')
    assert_true(expected in areas)


def test_scopus_url():
    expected = 'https://www.scopus.com/authid/detail.uri?\
partnerID=HzOxMe3b&authorId=7004212771&origin=inward'
    assert_equal(au.scopus_url, expected)


def test_subject_areas():
    areas = au.subject_areas
    assert_true(isinstance(areas, list))
    assert_true(len(areas) > 0)
    expected = ('Analytical Chemistry', 1, 'CHEM', '1602')
    # Frequency might differ
    assert_true(expected in areas)



import os
import requests
import sys
try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser

import scopus

config = ConfigParser()
config.optionxform = str
config.read(os.path.expanduser("~/.scopus/config"))


def download(url, params=None, accept="xml"):
    """Helper function to download a file and return its content.
    Parameters
    ----------
    url : string
        The URL to be parsed.

    params : dict (optional)
        Dictionary containing query parameters.  For required keys
        and accepted values see e.g.
        https://api.elsevier.com/documentation/AuthorRetrievalAPI.wadl

    accept : str (optional, default=xml)
        mime type of the file to be downloaded.  Accepted values are json,
        atom+xml, xml.

    Raises
    ------
    HTTPError
        If the status of the response is not ok.

    ValueError
        If the accept parameter is not one of the accepted values.

    Returns
    -------
    resp : byte-like object
        The content of the file, which needs to be serialized.

    Notes
    -----
    Loads the Authentication creditation into scopus namespace on first run.
    If there is a config file, which must contain InstToken, it is given
    preference.  Alternatively it loads the API key from my_scopus.py file.
    """
    accepted = ("json", "xml", "atom+xml")
    if accept.lower() not in accepted:
        raise ValueError('accept parameter must be one of ' +
                         ', '.join(accepted))
    if config.has_section('Authentication'):
        # Authentication with InstToken and Key from config file
        if not valid_config(config):
            msg = ("config file misspecified. It must contain an "
                   "Authentication section with two entries: APIKey "
                   "and InstToken. Please correct.")
            raise ValueError(msg)
        header = {'X-ELS-APIKey': config.get('Authentication', 'APIKey'),
                  'X-ELS-Insttoken': config.get('Authentication', 'InstToken')}
    else:
        # Authentication with Key from my_scopus.py file
        try:
            key = scopus.MY_API_KEY
        except AttributeError:
            load_api_key()  # loads MY_API_KEY in scopus namespace
        header = {'X-ELS-APIKey': scopus.MY_API_KEY}
    header.update({'Accept': 'application/{}'.format(accept)})
    resp = requests.get(url, headers=header, params=params)
    resp.raise_for_status()
    return resp


def get_content(qfile, refresh, *args, **kwds):
    """Helper function to read file content as xml.  The file is cached
    in a subfolder of ~/.scopus/.

    Parameters
    ----------
    qfile : string
        The name of the file to be created.

    refresh : bool
        Whether the file content should be refreshed if it exists.

    *args, **kwds : 
        Arguments and keywords to be passed on to download().

    Returns
    -------
    content : str
        The content of the file.
    """
    if not refresh and os.path.exists(qfile):
        with open(qfile, 'rb') as f:
            content = f.read()
    else:
        content = download(*args, **kwds).text.encode('utf-8')
        with open(qfile, 'wb') as f:
            f.write(content)
    return content


def load_api_key():
    """Helper function to create file with API Key on the first run.
    Function needs to be here because in __init__.py it causes problems
    with RTFD.io.
    """
    SCOPUS_API_FILE = os.path.expanduser("~/.scopus/my_scopus.py")
    with open(SCOPUS_API_FILE, "a+") as f:
        f.seek(0)
        exec(f.read(), globals())
        try:
            scopus.MY_API_KEY = MY_API_KEY
        except NameError:
            prompt = ("No API Key detected. Please enter a valid API Key "
                      "obtained from http://dev.elsevier.com/myapikey.html: \n")
            if sys.version_info >= (3, 0):
                key = input(prompt)
            else:
                key = raw_input(prompt)
            f.write('MY_API_KEY = "{}"'.format(key))
            scopus.MY_API_KEY = key


def valid_config(conf):
    """Test if config contains necessary sections."""
    return conf.has_option('Authentication', 'APIKey') and conf.has_option('Authentication', 'InstToken')



import os
import sys
from collections import namedtuple
from json import loads

from scopus.utils import get_content, get_encoded_text

ABSTRACT_RETRIEVAL_DIR = os.path.expanduser('~/.scopus/abstract_retrieval')

if not os.path.exists(ABSTRACT_RETRIEVAL_DIR):
    os.makedirs(ABSTRACT_RETRIEVAL_DIR)


class CitationOverview(object):

    @property
    def srctype(self):
        """Return the description of a record.
        Note: This may be empty.  You probably want the abstract instead.
        """
        return self.coredata.get('srctype')

    @property
    def startingPage(self):
        """Starting page."""
        return self.coredata.get('startingPage')

    @property
    def citedby_count(self):
        """Number of articles citing the abstract."""
        return self.coredata.get('citedby-count')

    @property
    def aggregationType(self):
        """Type of source the abstract is published in."""
        return self.coredata.get('aggregationType')

    @property
    def aggregationType(self):
        """Type of source the abstract is published in."""
        return self.coredata.get('creator')

    @property
    def pageRange(self):
        """Page range."""
        return 

    @property
    def endingPage(self):
        """Ending page."""
        return self.coredata.get('endingPage')

    @property
    def eid(self):
        """The EID of the abstract (might differ from the one provided)."""
        return self.coredata.get('eid')

    @property
    def pii(self):
        """The Publication Item Identifier (PII) of the abstract."""
        return self.coredata.get('pii')
    
    @property
    def source-id(self):
        """Scopus-ID of the source the abstract is published in."""
        return self.coredata.get('source-id')

    @property
    def description(self):
        """The abstract (description) of the abstract object."""
        return self.coredata.get('description')
    
    @property
    def issueIdentifier(self):
        """Issue number for abstract."""
        return self.coredata.get('issueIdentifier')

    @property
    def issn(self):
        """ISSN of the publisher.
        Note: If E-ISSN is known to Scopus, this returns both
        ISSN and E-ISSN in random order separated by blank space.
        """
        return self.coredata.get('issn')
    
    @property
    def title(self):
        """Abstract title."""
        return self.coredata.get('title')

    @property
    def volume(self):
        """Volume for the abstract."""
        return self.coredata.get('volume')

    @property
    def url(self):
        """URL to Abstract Retrieval API view of the abstract."""
        return self.coredata.get('url')

    @property
    def identifier(self):
        """Scopus-identifier for the abstract (the right part of the EID)."""
        return self.coredata.get('identifier')

    @property
    def doi(self):
        """Document Object Identifier (DOI) of the abstract."""
        return self.coredata.get('doi')
    
    @property
    def publicationName(self):
        """Name of source the abstract is published in (e.g. the Journal)."""
        return self.coredata.get('publicationName')

    @property
    def affiliations(self):
        """A list of namedtuples storing affilation information,
        where each namedtuple corresponds to one affilation.
        The information in each namedtuple is
        (name, id, country, city).  All entries are strings.
        """
        _out = []
        _order = 'name id country city'
        _aff = namedtuple('Affiliation', _order)
        for aff in self.affs:
            _new = _aff(name=aff.get('affilname'), id=aff.get('@id'),
                        country=aff.get('affiliation-country'),
                        city=aff.get('affiliation-city'))
            _out.append(_new)
        return _out

    @property
    def language(self):
        """The language code of the abstract."""
        return self.language.get('lang')


    @property
    def authors(self):
        """A list of namedtuples storing author information,
        where each namedtuple corresponds to one author.
        The information in each namedtuple is
        (name, surname, given_name, initials, id, affiliation_ids).
        All entries are strings, exept affiliation_ids,
        which is a list of strings.
        """
        _out = []
        _order = 'name surname given_name initials id affiliation_ids'
        _auth = namedtuple('Author', _order)
        for author in self.authors:
            author = {k.split(":", 1)[-1]: v for k, v in author.items()}
            try:
                _affs = [aff.get('@id') for aff in author.get('affiliation', {})]
            except AttributeError:
                _affs = [author.get('affiliation', {}).get('@id')]
            _new = _auth(name=author.get('indexed-name'),
                         surname=author.get('surname'),
                         given_name=author.get('given-name'),
                         initials=author.get('initials'),
                         id=author.get('@auid'), affiliation_ids=_affs)
            _out.append(_new)
        return _out

    @property
    def subjectAreas(self):
        """A list of namedtuples storing subject area information,
        where each namedtuple corresponds to one subject area.
        The information in each namedtuple is
        (name, abbreviation, code).  All entries are strings.
        """
        _out = []
        _order = 'name abbreviation code'
        _area = namedtuple('SubjectArea', _order)
        for area in areas:
            _new = _area(name=area.get('$'), abbreviation=area.get('@abbrev'),
                         code=area.get('code'))
            _out.append(_new)
        return _out

    @property
    def keywords(self):
        """A list of author provided keywords."""
        return [d.get('$') for d in self.keywords]

    def __init__(self, eid, refresh=False):
        """Class to represent the results from a Scopus Abstract Retrieval.

        Parameters
        ----------
        eid : str
            The EID of the abstract.

        refresh : bool (optional, default=False)
            Whether to refresh the cached file if it exists or not.

        Notes
        -----
        The files are cached in ~/.scopus/abstract_retrieval/{eid}.
        """
        # Get file content
        qfile = os.path.join(ABSTRACT_RETRIEVAL_DIR, eid)
        url = "https://api.elsevier.com/content/abstract/eid/{}".format(eid)
        res = get_content(qfile, url=url, refresh=refresh, accept='json')
        data = loads(res.decode('utf-8'))['abstracts-retrieval-response']

        # Coredata
        self.coredata = {k.split(":", 1)[-1]: v for k, v in data['coredata'].items()}
        # Affiliations
        self.affs = data['affiliation']
        # Authors
        self.authors = data['authors']['author']
        # Language
        self.language = {k.split(":", 1)[-1]: v for k, v in data['language'].items()}
        # Keywords
        self.keywords = data['authkeywords'].get('author-keyword', [])
        # Subject areas
        self.areas = data['subject-areas'].get('subject-area', [])
        # idxterms
        self.idxterms = data['idxterms']  # not used?
        # Item
        self.items = data['item']['bibrecord']



import os
import sys
import xml.etree.ElementTree as ET

from scopus.utils import download, ns
from scopus.scopus_api import ScopusAbstract

SCOPUS_SEARCH_DIR = os.path.expanduser('~/.scopus/search')

if not os.path.exists(SCOPUS_SEARCH_DIR):
    os.makedirs(SCOPUS_SEARCH_DIR)


class ScopusSearch(object):
    @property
    def EIDS(self):
        """List of EIDs retrieved."""
        return self._EIDS

    def __init__(self, query, fields='eid', count=200, start=0,
                 max_entries=5000, refresh=False):
        """Class to search a query, and retrieve a list of EIDs as results.

        Parameters
        ----------
        query : str
            A string of the query.

        fields : str (optional, default='eid')
            The fields you want returned.  Allowed fields are specified in
            https://dev.elsevier.com/guides/ScopusSearchViews.htm.  Since
            currently only EIDs are stored, this parameter is being kept
            for later use only.

        count : int (optional, default=200)
            The number of entries to be displayed at once.  A smaller number
            means more queries with each query having less results.

        start : int (optional, default=0)
            The entry number of the first search item to start with.

        refresh : bool (optional, default=False)
            Whether to refresh the cached file if it exists or not.

        max_entries : int (optional, default=5000)
            Raise error when the number of results is beyond this number.
            The Scopus Search Engine does not allow more than 5000 entries.

        Raises
        ------
        Exception
            If the number of search results exceeds max_entries.

        Notes
        -----
        XML results are cached in ~/.scopus/search/{query}.

        The EIDs are stored as a property named EIDS.
        """

        self.query = query
        qfile = os.path.join(SCOPUS_SEARCH_DIR,
                             # We need to remove / in a DOI here so we can save
                             # it as a file.
                             query.replace('/', '_slash_'))

        if os.path.exists(qfile) and not refresh:
            with open(qfile) as f:
                self._EIDS = [eid for eid in
                              f.read().strip().split('\n')
                              if eid]
        else:
            # No cached file exists, or we are refreshing.
            # First, we get a count of how many things to retrieve
            url = 'https://api.elsevier.com/content/search/scopus'
            params = {'query': query, 'field': fields, 'count': 0, 'start': 0}
            xml = download(url=url, params=params).text.encode('utf-8')
            results = ET.fromstring(xml)

            N = results.find('opensearch:totalResults', ns)
            try:
                N = int(N.text)
            except:
                N = 0

            if N > max_entries:
                raise Exception(('N = {}. '
                                 'Set max_entries to a higher number or '
                                 'change your query ({})').format(N, query))

            self._EIDS = []
            while N > 0:
                params = {'query': query, 'fields': fields,
                          'count': count, 'start': start}
                resp = download(url=url, params=params, accept="json")
                results = resp.json()

                if 'entry' in results.get('search-results', []):
                    self._EIDS += [str(r['eid']) for
                                   r in results['search-results']['entry']]
                start += count
                N -= count

            with open(qfile, 'wb') as f:
                for eid in self.EIDS:
                    f.write('{}\n'.format(eid).encode('utf-8'))

    def __str__(self):
        s = """{query}
        Resulted in {N} hits.
    {entries}"""
        return s.format(query=self.query,
                        N=len(self.EIDS),
                        entries='\n    '.join(self.EIDS))

    @property
    def org_summary(self):
        """Summary of search results."""
        s = ''
        for i, eid in enumerate(self.EIDS):
            abstract = ScopusAbstract(eid)
            if abstract.aggregationType == 'Journal':
                s += '{0}. {1}\n'.format(i + 1, abstract)
        return s



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for `ScopusAffiliation` module."""

from nose.tools import assert_equal, assert_true

import scopus


aff = scopus.ScopusAffiliation('60000356', refresh=True)


def test_address():
    assert_equal(aff.address, 'Private Bag X3')


def test_affiliation_id():
    assert_equal(aff.affiliation_id, '60000356')


def test_city():
    assert_equal(aff.city, 'Cape Town')


def test_country():
    assert_equal(aff.country, 'South Africa')


def test_date_created():
    assert_equal(aff.date_created, (2008, 2, 2))


def test_name():
    assert_equal(aff.name, 'University of Cape Town')


def test_nauthors():
    expected = '10473'
    assert_true(aff.nauthors >= expected)


def test_ndocuments():
    expected = '47841'
    assert_true(aff.ndocuments >= expected)


def test_org_type():
    assert_equal(aff.org_type, 'univ')


def test_org_domain():
    assert_equal(aff.org_domain, 'uct.ac.za')


def test_org_url():
    assert_equal(aff.org_url, 'http://www.uct.ac.za')


def test_url():
    expected = 'https://www.scopus.com/affil/profile.uri?afid=\
60000356&partnerID=HzOxMe3b&origin=inward'
    assert_equal(aff.url, expected)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for `AuthorSearch` module."""

from collections import namedtuple

from nose.tools import assert_equal

import scopus

s = scopus.AuthorSearch('authlast(selten) and authfirst(reinhard)',
                        refresh=True)


def test_authors():
    order = 'eid surname initials givenname affiliation documents '\
            'affiliation_id city country areas'
    Author = namedtuple('Author', order)
    expected = [Author(eid='9-s2.0-6602907525', surname='Selten',
                initials='R.', givenname='Reinhard',
                affiliation='Universitat Bonn', documents='73',
                affiliation_id='60007493', city='Bonn', country='Germany',
                areas='ECON (71); MATH (19); BUSI (15)')]
    assert_equal(s.authors, expected)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for `ScopusAbstract` module."""

from nose.tools import assert_equal, assert_true

import scopus


ab = scopus.ScopusAbstract("2-s2.0-84930616647", view="FULL", refresh=True)


def test_affiliations():
    affs = ab.affiliations
    assert_true(len(affs) == 1)
    aff = affs[0]
    assert_true(isinstance(aff, scopus.scopus_api._ScopusAffiliation))
    assert_equal(aff.id, '60027950')
    assert_equal(aff.affilname, 'Carnegie Mellon University')
    assert_equal(aff.city, 'Pittsburgh')
    assert_equal(aff.country, 'United States')
    link = 'https://api.elsevier.com/content/affiliation/affiliation_id/60027950'
    assert_equal(aff.href, link)


def test_aggregationType():
    assert_equal(ab.aggregationType, 'Journal')


def test_article_number():
    pass


def test_authors():
    assert_true(isinstance(ab.authors[0], scopus.scopus_api._ScopusAuthor))


def test_bibtex():
    expected = '@article{Kitchin2015ExamplesPublishing,\n  author = \
{John R. Kitchin},\n  title = {Examples of effective data sharing in \
scientific publishing},\n  journal = {ACS Catalysis},\n  year = {2015},\n  \
volume = {5},\n  number = {6},\n  pages = {3894-3899},\n  doi = {10.1021/\
acscatal.5b00538}\n}\n\n'
    assert_equal(ab.bibtex, expected)


def test_citationLanguage():
    assert_equal(ab.citationLanguage, 'English')


def test_citationType():
    assert_equal(ab.citationType, 're')


def test_citedby_count():
    expected = 5
    assert_true(ab.citedby_count >= expected)


def test_coverDate():
    assert_equal(ab.coverDate, '2015-06-05')


def test_doi():
    assert_equal(ab.doi, '10.1021/acscatal.5b00538')


def test_endingPage():
    assert_equal(ab.endingPage, '3899')


def test_html():
    expected = '<a href="https://www.scopus.com/authid/detail.url?origin=\
AuthorProfile&authorId=7004212771">John R. Kitchin</a>, <a href="https://www.\
scopus.com/inward/record.uri?partnerID=HzOxMe3b&scp=84930616647&\
origin=inward">Examples of effective data sharing in scientific \
publishing</a>, <a href="https://www.scopus.com/source/sourceInfo.url?\
sourceId=19700188320">ACS Catalysis</a>, <b>5(6)</b>, p. 3894-3899, \
(2015-06-05). <a href="https://doi.org/10.1021/acscatal.5b00538">\
doi:10.1021/acscatal.5b00538</a>.'
    assert_equal(ab.html, expected)


def test_issn():
    assert_equal(ab.issn, '21555435')


def test_issueIdentifier():
    assert_equal(ab.issueIdentifier, '6')


def test_latex():
    expected = 'John R. Kitchin, \\textit{Examples of effective data sharing \
in scientific publishing}, ACS Catalysis, \\textbf{5(6)}, p. 3894-3899, \
(2015-06-05). \\href{https://doi.org/10.1021/acscatal.5b00538}{doi:10.1021/\
acscatal.5b00538}, \\href{https://www.scopus.com/inward/record.uri?partnerID=\
HzOxMe3b&scp=84930616647&origin=inward}{scopus:2-s2.0-84930616647}.'
    assert_equal(ab.latex, expected)


def test_nauthors():
    assert_equal(ab.nauthors, 1)


def test_pageRange():
    assert_equal(ab.pageRange, '3894-3899')


def test_publicationName():
    assert_equal(ab.publicationName, 'ACS Catalysis')


def test_publisher():
    assert_equal(ab.publisher, '\nAmerican Chemical Society\nservice@acs.org\n')


def test_refcount():
    assert_equal(ab.refcount, '22')


def test_references():
    expected = [
        '2-s2.0-84881394200', '2-s2.0-84896585411', '2-s2.0-84949115648',
        '2-s2.0-84908637059', '2-s2.0-84901638552', '2-s2.0-84896380535',
        '2-s2.0-84923164062', '2-s2.0-84923164062', '2-s2.0-84930667693',
        '2-s2.0-79952591087', '2-s2.0-84923165709', '2-s2.0-0036572216',
        '2-s2.0-84924117832', '2-s2.0-84930624433', '2-s2.0-79955561198',
        '2-s2.0-84930642229', '2-s2.0-0010630518', '2-s2.0-84861337169',
        '2-s2.0-34247481878', '2-s2.0-79958260504', '2-s2.0-58149108944',
        '2-s2.0-84917679308']
    assert_equal(ab.references, expected)


def test_ris():
    expected = 'TY  - JOUR\n\
TI  - Examples of effective data sharing in scientific publishing\n\
JO  - ACS Catalysis\nVL  - 5\nDA  - 2015-06-05\nSP  - 3894-3899\n\
PY  - 2015\nDO  - 10.1021/acscatal.5b00538\nUR  - https://doi.org/10.1021/\
acscatal.5b00538\nAU  - Kitchin J.R.\nIS  - 6\nER  - \n\n'
    assert_equal(ab.ris, expected)


def test_source_id():
    assert_equal(ab.source_id, '19700188320')


def test_srctype():
    assert_equal(ab.srctype, 'j')


def test_startingPage():
    assert_equal(ab.startingPage, '3894')


def test_subjectAreas():
    assert_equal(ab.subjectAreas, ['Catalysis'])


def test_title():
    expected = 'Examples of effective data sharing in scientific publishing'
    assert_equal(ab.title, expected)


def test_url():
    expected = 'https://api.elsevier.com/content/abstract/scopus_id/84930616647'
    assert_equal(ab.url, expected)


def test_volume():
    assert_equal(ab.volume, '5')


def test_website():
    assert_equal(ab.website, 'http://pubs.acs.org/page/accacs/about.html')



import matplotlib.pyplot as plt
from operator import itemgetter

from .scopus_api import ScopusAbstract, ScopusJournal
from .scopus_author import ScopusAuthor


def report(scopus_search, label):
    """Print out an org-mode report for search results.

    Parameters
    ----------
    scopus_search : scopus.scopus_search.ScopusSearch
        An object resulting from a ScopusSearch.

    label : str
        The label used in the document title ("Report for ...").
    """

    counts = {}  # to count papers per author
    journals = {}  # to count publications per journal
    author_count = []  # to count a paper's number of authors for a histogram
    paper_cites = {}
    Ncites = 0
    document_types = {}

    papers = 0  # to count number of publications

    for eid in scopus_search.EIDS:
        a = ScopusAbstract(eid)

        # Get types of documents
        try:
            document_types[a.aggregationType] += 1
        except KeyError:
            document_types[a.aggregationType] = 1

        if a.aggregationType == 'Journal':
            Ncites += int(a.citedby_count)  # get total cites
            papers += 1

            # get count for journals
            jkey = (a.publicationName, a.source_id, a.issn)
            try:
                journals[jkey] += 1
            except KeyError:
                journals[jkey] = 1

            # get authors per paper
            author_count += [len(a.authors)]

            # now count papers per author
            for author in a.authors:
                key = (author.indexed_name, author.auid)
                try:
                    counts[key] += 1
                except KeyError:
                    counts[key] = 1

            # counting cites per paper
            key = (a.title, a.scopus_link)
            try:
                paper_cites[key] += a.citedby_count
            except KeyError:
                paper_cites[key] = a.citedby_count

    print('*** Report for {}\n'.format(label))
    print('#+attr_latex: :placement [H] :center nil')
    print('#+caption: Types of documents found for {}.'.format(label))
    print('| Document type | count |\n|-')
    for key in document_types:
        print('| {0} | {1} |'.format(key, document_types[key]))

    print('\n\n')
    print('{0} articles ({2} citations) '
          'found by {1} authors'.format(papers, len(counts), Ncites))

    # Author counts {(name, scopus-id): count}
    view = [('[[https://www.scopus.com/authid/detail.uri?authorId={0}][{1}]]'.format(
             k[1], k[0]),  # org-mode link
            v, k[1]) for k, v in counts.items()] # counts, scopus-id
    view.sort(reverse=True, key=itemgetter(1))

    print('\n#+attr_latex: :placement [H] :center nil')
    print('#+caption: Author publication counts for {0}.'.format(label))
    print('| name | count | categories |')
    print('|-')
    for name, count, scopus_id in view[0:20]:
        cats = ', '.join(['{0} ({1})'.format(cat[0], cat[1])
                          for cat in ScopusAuthor(scopus_id).categories[0:3]])
        print('| {0} | {1} | {2} |'.format(name, count, cats))

    # journal view
    s = '[[https://www.scopus.com/source/sourceInfo.url?sourceId={0}][{1}]]'
    jview = [(s.format(k[1], k[0][0:50]),  # url
              k[1],  # source_id
              k[2],  # issn
              v)  # count
             for k, v in journals.items()]
    jview.sort(reverse=True, key=itemgetter(3))

    print('\n\n')
    print('#+attr_latex: :placement [H] :center nil')
    print('#+caption: Journal publication counts for {0}.'.format(label))
    print('| Journal | count | IPP |')
    print('|-')

    for journal, sid, issn, count in jview[0:12]:
        # issn may contain E-ISSN
        issn_tokens = issn.split()
        try:
            JOURNAL = ScopusJournal(issn_tokens[0])
        except:
            JOURNAL = ScopusJournal(issn_tokens[1])
        IPP = JOURNAL.IPP or 0
        print('| {0} | {1} | {2} |'.format(journal, count, IPP))

    # view of journals sorted by `IPP
    JVIEW = []
    for journal, sid, issn, count in jview:
        issn_tokens = issn.split()
        try:
            JOURNAL = ScopusJournal(issn_tokens[0])
        except:
            JOURNAL = ScopusJournal(issn_tokens[1])
        IPP = JOURNAL.IPP or 0
        JVIEW.append([journal, count, IPP])
    JVIEW.sort(reverse=True, key=itemgetter(2))

    print('\n\n')
    print('#+attr_latex: :placement [H] :center nil')
    print('#+caption: Journal publication counts'
          ' for {0} sorted by IPP.'.format(label))
    print('| Journal | count | IPP |')
    print('|-')
    for journal, count, IPP in JVIEW[0:12]:
        print('|{0}|{1}|{2}|'.format(journal, count, IPP))

    # top cited papers
    pview = [('[[{0}][{1}]]'.format(k[1], k[0][0:60]),
              int(v))
             for k, v in paper_cites.items()]
    pview.sort(reverse=True, key=itemgetter(1))

    # Compute department j-index
    hindex = 0
    for i, entry in enumerate(pview):
        # entry is url, source_id, count
        u, count = entry
        if count > i + 1:
            continue
        else:
            hindex = i + 1
            break

    print('\n\n#+attr_latex: :placement [H] :center nil')
    print('#+caption: Top cited publication'
          ' counts for {0}. j-index = {1}.'.format(label, hindex))
    print('| title | cite count |\n|-')
    for title, count in pview[0:10]:
        print('| {0} | {1} |'.format(title, count))

    plt.figure()
    plt.hist(author_count, 20)
    plt.xlabel('# authors')
    plt.ylabel('frequency')
    plt.savefig('{0}-nauthors-per-publication.png'.format(label))

    print('\n\n#+caption: Number of authors '
          'on each publication for {}.'.format(label))
    print('[[./{0}-nauthors-per-publication.png]]'.format(label))
    print('''**** Bibliography  :noexport:
     :PROPERTIES:
     :VISIBILITY: folded
     :END:''')
    print(scopus_search.org_summary)



import os
import sys
import xml.etree.ElementTree as ET

from scopus.utils import get_content, get_encoded_text, ns

SCOPUS_XML_DIR = os.path.expanduser('~/.scopus/xml')
SCOPUS_ISSN_DIR = os.path.expanduser('~/.scopus/issn')

if not os.path.exists(SCOPUS_XML_DIR):
    os.makedirs(SCOPUS_XML_DIR)

if not os.path.exists(SCOPUS_ISSN_DIR):
    os.makedirs(SCOPUS_ISSN_DIR)


class ScopusAbstract(object):
    @property
    def url(self):
        """URL to the abstract."""
        return self._url

    @property
    def doi(self):
        """DOI of article."""
        return self._doi

    @property
    def website(self):
        """Website of article."""
        return self._website

    @property
    def title(self):
        """Abstract title."""
        return self._title

    @property
    def publicationName(self):
        """Name of source the abstract is published in."""
        return self._publicationName

    @property
    def aggregationType(self):
        """Type of source the abstract is published in."""
        return self._aggregationType

    @property
    def srctype(self):
        """Type (short version) of source the abstract is published in."""
        return self._srctype

    @property
    def citationType(self):
        """Type (short version) of the article."""
        return self._citationType

    @property
    def citationLanguage(self):
        """Language of the article."""
        return self._citationLang

    @property
    def citedby_count(self):
        """Number of articles citing the abstract."""
        return int(self._citedby_count)

    @property
    def publisher(self):
        """Name of the publisher of the abstract."""
        return self._publisher

    @property
    def source_id(self):
        """Scopus source_id of the abstract."""
        return self._source_id

    @property
    def issn(self):
        """ISSN of the publisher.
        Note: If E-ISSN is known to Scopus, this returns both
        ISSN and E-ISSN in random order separated by blank space.
        """
        return self._issn

    @property
    def volume(self):
        """Volume for the abstract."""
        return self._volume

    @property
    def issueIdentifier(self):
        """Issue number for abstract."""
        return self._issueIdentifier

    @property
    def article_number(self):
        """Article number."""
        return self._article_number

    @property
    def startingPage(self):
        """Starting page."""
        return self._startingPage

    @property
    def endingPage(self):
        """Ending page."""
        return self._endingPage

    @property
    def pageRange(self):
        """Page range."""
        return self._pageRange

    @property
    def coverDate(self):
        """The date of the cover the abstract is in."""
        return self._coverDate

    @property
    def subjectAreas(self):
        """List of subject areas of article.
        Note: Requires the FULL view of the article.
        """
        return self._subjectAreas

    @property
    def authors(self):
        """A list of scopus_api._ScopusAuthor objects."""
        return self._authors

    @property
    def affiliations(self):
        """A list of scopus_api._ScopusAffiliation objects."""
        return self._affiliations

    @property
    def nauthors(self):
        """Return number of authors listed in the abstract."""
        return len(self.authors)

    @property
    def refcount(self):
        """Number of references of an article.
        Note: Requires the FULL view of the article.
        """
        if self._references is not None:
            return self._references.attrib['refcount']
        else:
            return None

    @property
    def references(self):
        """Return EIDs of references of an article.
        Note: Requires the FULL view of the article.
        """
        if self._references is not None:
            eids = [r.find("ref-info/refd-itemidlist/itemid", ns).text for r
                    in self._references.findall("reference", ns)]
            return ["2-s2.0-" + eid for eid in eids]
        else:
            return None

    @property
    def abstract(self):
        """Return the abstract of an article."""
        return self._abstract

    @property
    def description(self):
        """Return the description of a record.
        Note: This may be empty.  You probably want the abstract instead.
        """
        return self._description

    def __init__(self, EID, view='META_ABS', refresh=False):
        """Class to represent the results from a Scopus abstract.

        Parameters
        ----------
        EID : str
            The Scopus ID (EID) of an abstract.

        view : str (optional, default=META_ABS)
            The view of the file that should be downloaded.  Will not take
            effect for already cached files. Supported values: META, META_ABS,
            FULL, where FULL includes all information of META_ABS view and
            META_ABS includes all information of the META view .  See
            https://dev.elsevier.com/guides/AbstractRetrievalViews.htm
            for details.

        refresh : bool (optional, default=False)
            Whether to refresh the cached file if it exists or not.

        Notes
        -----
        The files are cached in ~/.scopus/xml/{eid}.
        """
        allowed_views = ('META', 'META_ABS', 'FULL')
        if view not in allowed_views:
            raise ValueError('view parameter must be one of ' +
                             ', '.join(allowed_views))

        # Get file content
        qfile = os.path.join(SCOPUS_XML_DIR, EID)
        url = "https://api.elsevier.com/content/abstract/eid/{}".format(EID)
        params = {'view': view}
        xml = ET.fromstring(get_content(qfile, url=url, refresh=refresh,
                                        params=params))
        # Remove default namespace if present
        remove = u'{http://www.elsevier.com/xml/svapi/abstract/dtd}'
        nsl = len(remove)
        for elem in xml.getiterator():
            if elem.tag.startswith(remove):
                elem.tag = elem.tag[nsl:]

        self.xml = xml
        if xml.tag == 'service-error':
            raise Exception('\n{0}\n{1}'.format(EID, self.xml))

        # Parse coredata
        coredata = xml.find('coredata', ns)
        self._url = get_encoded_text(coredata, 'prism:url')
        self.identifier = get_encoded_text(coredata, 'dc:identifier')
        self.eid = get_encoded_text(coredata, 'eid')
        self._doi = get_encoded_text(coredata, 'prism:doi')
        self._title = get_encoded_text(coredata, 'dc:title')
        self._aggregationType = get_encoded_text(coredata,
                                                 'prism:aggregationType')
        self._publicationName = get_encoded_text(coredata,
                                                 'prism:publicationName')
        self._srctype = get_encoded_text(coredata, 'srctype')
        self._citedby_count = get_encoded_text(coredata, 'citedby-count')
        self._publisher = get_encoded_text(coredata, 'dc:publisher')
        self._source_id = get_encoded_text(coredata, 'source-id')
        self._issn = get_encoded_text(coredata, 'prism:issn')
        self._volume = get_encoded_text(coredata, 'prism:volume')
        self._issueIdentifier = get_encoded_text(coredata,
                                                 'prism:issueIdentifier')
        self._article_number = get_encoded_text(coredata, 'article-number')
        self._startingPage = get_encoded_text(coredata, 'prism:startingPage')
        self._endingPage = get_encoded_text(coredata, 'prism:endingPage')
        self._pageRange = get_encoded_text(coredata, 'prism:pageRange')
        self._coverDate = get_encoded_text(coredata, 'prism:coverDate')
        self.creator = get_encoded_text(coredata, 'dc:creator')
        self._description = get_encoded_text(coredata, 'dc:description')
        self._abstract = get_encoded_text(coredata,
                                          'dc:description/abstract/ce:para')

        self.scopus_link = coredata.find('link[@rel="scopus"]', ns).get('href')
        self.self_link = coredata.find('link[@rel="self"]', ns).get('href')
        cite_link = coredata.find('link[@rel="cited-by"]', ns)
        if cite_link:
            cite_link = cite_link.get('href')
        self.cite_link = cite_link

        # Parse subject-areas
        subjectAreas = xml.find('subject-areas', ns)
        try:
            self._subjectAreas = [a.text for a in subjectAreas]
        except:
            self._subjectAreas = None

        # Parse authors
        authors = xml.find('authors', ns)
        try:
            self._authors = [_ScopusAuthor(author) for author in authors]
        except TypeError:
            self._authors = None
        self._affiliations = [_ScopusAffiliation(aff) for aff
                              in xml.findall('affiliation', ns)]

        # Parse items
        items = xml.find('item', ns)
        self._website = get_encoded_text(
            items, 'bibrecord/head/source/website/ce:e-address')
        try:
            self._citationType = items.find(
                'bibrecord/head/citation-info/citation-type').get("code")
        except:
            self._citationType = None
        try:
            self._citationLang = items.find(
                'bibrecord/head/citation-info/citation-language').get("language")
        except:
            self._citationLang = None
        try:
            self._references = items.find('bibrecord/tail/bibliography', ns)
        except:
            self._references = None

    # def get_corresponding_author_info(self):
    #     """Try to get corresponding author information.
    #     Returns (scopus-id, name, email).

    #     This may not work anymore. Scopus may be hiding the email address now
    #     from scrapers like this one ;)

    #     <a href="http://www.scopus.com/authid/detail.url?authorId=56273017000&amp;amp;eid=2-s2.0-84927602604" title="Show Author Details">Zhang, Z.-J.</a><a role="presentation" href="#" aria-labelledby="superscript_c"><sup>c</sup></a><img src="http://www.scopus.com/static/images/s.gif" class="" border="0" height="0" width="4" alt="" title=""><a href="/cdn-cgi/l/email-protection#1c66747d727b6674757669725c6f746932797869327f72" OnClick="authorEmailEvent(this,'abstract')" title="Email to this author" class="correspondenceEmail">&nbsp;</a><font>,&nbsp;</font>

    #     """
    #     resp = requests.get(self.scopus_link)
    #     import re
    #     m = re.search('<a href="http://www.scopus.com/authid/detail.url'
    #                   '\?authorId=([0-9]*)[^"]*"'  # group 1 has scopus id
    #                    # group 2 has author name
    #                   ' title="Show Author Details">([^<]*)'
    #                   '</a>.*'
    #                   # group 3 has email address
    #                   '<a href="mailto:([^"]*)".*class="correspondenceEmail">',
    #                   resp.text.encode('utf-8'))
    #     if m:
    #         return (m.group(1), m.group(2), m.group(3))
    #     else:
    #         return (None, None, None)

    def get_corresponding_author_info(self):
        """Try to get corresponding author information.

        Returns (scopus-id, name, email).
        """
        resp = requests.get(self.scopus_link)
        from lxml import html

        parsed_doc = html.fromstring(resp.content)
        for div in parsed_doc.body.xpath('.//div'):
            for a in div.xpath('a'):
                if '/cdn-cgi/l/email-protection' in a.get('href', ''):
                    encoded_text = a.attrib['href'].replace('/cdn-cgi/l/email-protection#', '')
                    key = int(encoded_text[0:2], 16)
                    email = ''.join([chr(int('0x{}'.format(x), 16) ^ key)
                                     for x in
                                     map(''.join, zip(*[iter(encoded_text[2:])]*2))])
                    for aa in div.xpath('a'):
                        if 'http://www.scopus.com/authid/detail.url' in aa.get('href', ''):
                            scopus_url = aa.attrib['href']
                            name = aa.text
                        else:
                            scopus_url, name = None, None

        return (scopus_url, name, email)

    def __str__(self):
        """Return pretty text version of the abstract.

        Assumes the abstract is a journal article and was loaded with
        view="META_ABS" or view="FULL".
        """

        if len(self.authors) > 1:
            authors = ', '.join([str(a.initials) +
                                 ' ' +
                                 str(a.surname)
                                 for a in self.authors[0:-1]])
            authors += (' and ' +
                        str(self.authors[-1].initials) +
                        ' ' + str(self.authors[-1].surname))
        else:
            a = self.authors[0]
            authors = str(a.given_name) + ' ' + str(a.surname)

        s = '[[{self.scopus_link}][{self.eid}]]  '
        s += '{authors}, {self.title}, {self.publicationName}, '
        s += '{self.volume}'
        if self.issueIdentifier:
            s += '({self.issueIdentifier}), '
        else:
            s += ', '
        if self.pageRange:
            s += 'p. {self.pageRange}, '
        elif self.startingPage:
            s += 'p. {self.startingPage}, '
        elif self.article_number:
            s += 'Art. No. {self.article_number} '
        else:
            s += '(no pages found) '

        from dateutil.parser import parse
        pubDate = parse(self.coverDate)

        s += '({}).'.format(pubDate.year)
        s += ' https://doi.org/{self.doi},'
        s += ' {self.scopus_link},'
        s += ' cited {self.citedby_count} times (Scopus).\n'
        s += '  Affiliations:\n   '
        s += '\n   '.join([str(aff) for aff in self.affiliations])

        return s.format(authors=authors,
                        self=self)

    @property
    def latex(self):
        """Return LaTeX representation of the abstract."""
        s = ('{authors}, \\textit{{{title}}}, {journal}, {volissue}, '
             '{pages}, ({date}). {doi}, {scopus_url}.')
        if len(self.authors) > 1:
            authors = ', '.join([str(a.given_name) +
                                 ' ' + str(a.surname)
                                 for a in self.authors[0:-1]])
            authors += (' and ' +
                        str(self.authors[-1].given_name) +
                        ' ' + str(self.authors[-1].surname))
        else:
            a = self.authors[0]
            authors = str(a.given_name) + ' ' + str(a.surname)
        title = self.title
        journal = self.publicationName
        volume = self.volume
        issue = self.issueIdentifier
        if volume and issue:
            volissue = '\\textbf{{{0}({1})}}'.format(volume, issue)
        elif volume:
            volissue = '\\textbf{{0}}'.format(volume)
        else:
            volissue = 'no volume'
        date = self.coverDate
        if self.pageRange:
            pages = 'p. {0}'.format(self.pageRange)
        elif self.startingPage:
            pages = 'p. {self.startingPage}'.format(self)
        elif self.article_number:
            pages = 'Art. No. {self.article_number}, '.format(self)
        else:
            pages = '(no pages found)'
        doi = '\\href{{https://doi.org/{0}}}{{doi:{0}}}'.format(self.doi)
        scopus_url = '\href{{{0}}}{{scopus:{1}}}'.format(self.scopus_link,
                                                         self.eid)

        return s.format(**locals())

    @property
    def html(self):
        """Returns an HTML citation."""
        s = (u'{authors}, {title}, {journal}, {volissue}, {pages}, '
             '({date}). {doi}.')

        au_link = ('<a href="https://www.scopus.com/authid/detail.url'
                   '?origin=AuthorProfile&authorId={0}">{1}</a>')

        if len(self.authors) > 1:
            authors = u', '.join([au_link.format(a.auid,
                                                (str(a.given_name) +
                                                 ' ' + str(a.surname)))
                                 for a in self.authors[0:-1]])
            authors += (u' and ' +
                        au_link.format(self.authors[-1].auid,
                                       (str(self.authors[-1].given_name) +
                                        ' ' +
                                        str(self.authors[-1].surname))))
        else:
            a = self.authors[0]
            authors = au_link.format(a.auid,
                                     str(a.given_name) + ' ' + str(a.surname))

        title = u'<a href="{link}">{title}</a>'.format(link=self.scopus_link,
                                                      title=self.title)

        jname = self.publicationName
        sid = self.source_id
        jlink = ('<a href="https://www.scopus.com/source/sourceInfo.url'
                 '?sourceId={sid}">{journal}</a>')
        journal = jlink.format(sid=sid, journal=jname)

        volume = self.volume
        issue = self.issueIdentifier
        if volume and issue:
            volissue = u'<b>{0}({1})</b>'.format(volume, issue)
        elif volume:
            volissue = u'<b>{0}</b>'.format(volume)
        else:
            volissue = 'no volume'
        date = self.coverDate
        if self.pageRange:
            pages = u'p. {0}'.format(self.pageRange)
        elif self.startingPage:
            pages = u'p. {self.startingPage}'.format(self=self)
        elif self.article_number:
            pages = u'Art. No. {self.article_number}, '.format(self=self)
        else:
            pages = '(no pages found)'
        doi = '<a href="https://doi.org/{0}">doi:{0}</a>'.format(self.doi)

        html = s.format(**locals())
        return html.replace('None', '')

    @property
    def bibtex(self):
        """Bibliographic entry in BibTeX format.

        Returns
        -------
        bibtex : str
            A string representing a bibtex entry for the item.

        Raises
        ------
        ValueError : If the item's aggregationType is not Journal.
        """
        if self.aggregationType != 'Journal':
            raise ValueError('Only Journal articles supported.')
        template = u'''@article{{{key},
  author = {{{author}}},
  title = {{{title}}},
  journal = {{{journal}}},
  year = {{{year}}},
  volume = {{{volume}}},
  number = {{{number}}},
  pages = {{{pages}}},
  doi = {{{doi}}}
}}

'''
        if self.pageRange:
            pages = self.pageRange
        elif self.startingPage:
            pages = self.startingPage
        elif self.article_number:
            pages = self.article_number
        else:
            pages = 'no pages found'
        year = self.coverDate[0:4]
        first = self.title.split()[0].title()
        last = self.title.split()[-1].title()
        key = ''.join([self.authors[0].surname, year, first, last])
        authors = ' and '.join(["{} {}".format(a.given_name, a.surname)
                                for a in self.authors])
        bibtex = template.format(
            key=key, author=authors, title=self.title,
            journal=self.publicationName, year=year, volume=self.volume,
            number=self.issueIdentifier, pages=pages, doi=self.doi)
        return bibtex

    @property
    def ris(self):
        """Bibliographic entry in RIS (Research Information System Format)
        format.

        Returns
        -------
        ris : str
            The RIS string representing an item.

        Raises
        ------
        ValueError : If the item's aggregationType is not Journal.
        """
        if self.aggregationType != 'Journal':
            raise ValueError('Only Journal articles supported.')
        template = u'''TY  - JOUR
TI  - {title}
JO  - {journal}
VL  - {volume}
DA  - {date}
SP  - {pages}
PY  - {year}
DO  - {doi}
UR  - https://doi.org/{doi}
'''
        ris = template.format(
            title=self.title, journal=self.publicationName,
            volume=self.volume, date=self.coverDate, pages=self.pageRange,
            year=self.coverDate[0:4], doi=self.doi)
        for au in self.authors:
            ris += 'AU  - {}\n'.format(au.indexed_name)
        if self.issueIdentifier is not None:
            ris += 'IS  - {}\n'.format(self.issueIdentifier)
        ris += 'ER  - \n\n'
        return ris


class _ScopusAuthor(object):
    """An internal class for a author in a ScopusAbstract."""
    def __init__(self, author):
        """author should be an xml element.
        The following attributes are supported:

        author
        indexed_name
        given_name
        surname
        initials
        author_url - the scopus api url to get more information
        auid - the scopus id for the author
        scopusid - the scopus id for the author
        seq - the index of the author in the author list.
        affiliations - a list of ScopusAuthorAffiliation objects

        This class is not the same as the one in scopus.scopus_author, which
        uses the scopus author api.

        """
        self.author = author
        self.indexed_name = get_encoded_text(author, 'ce:indexed-name')
        self.given_name = get_encoded_text(author, 'ce:given-name')
        self.surname = get_encoded_text(author, 'ce:surname')
        self.initials = get_encoded_text(author, 'ce:initials')
        self.author_url = get_encoded_text(author, 'author-url')
        self.auid = author.attrib.get('auid')
        self.scopusid = self.auid
        self.seq = author.attrib.get('seq')
        self.affiliations = [_ScopusAuthorAffiliation(aff)
                             for aff in author.findall('affiliation', ns)]

    def __str__(self):
        s = """{0.seq}. {0.given_name} {0.surname} scopusid:{0.auid} """
        s += ' '.join([str(aff) for aff in self.affiliations])
        return s.format(self)


class _ScopusAffiliation(object):
    """Internal class to represent the affiliations in an Abstract."""
    def __init__(self, affiliation):
        """affiliation should be an xml element from the main abstract"""
        self.affiliation = affiliation
        self.affilname = get_encoded_text(affiliation, 'affilname')
        self.city = get_encoded_text(affiliation, 'affiliation-city')
        self.country = get_encoded_text(affiliation, 'affiliation-country')
        self.href = affiliation.attrib.get('href', None)
        self.id = affiliation.attrib.get('id', None)

    def __str__(self):
        return 'id:{0.id} {0.affilname}'.format(self)


class _ScopusAuthorAffiliation(object):
    """Internal class to represent the affiliation in an Author element"""
    def __init__(self, affiliation):
        """affiliation should be an xml element from an Author element."""
        self.affiliation = affiliation
        self.id = affiliation.get('id', None)
        self.href = affiliation.get('href', None)

    def __str__(self):
        return 'affiliation_id:{0.id}'.format(self)


class ScopusJournal(object):
    """Class to represent a journal from the Scopus API."""

    def __init__(self, ISSN, refresh=False):
        ISSN = str(ISSN)
        self.issn = ISSN

        qfile = os.path.join(SCOPUS_ISSN_DIR, ISSN)
        url = ("https://api.elsevier.com/content/serial/title/issn:" + ISSN)
        self.xml = ET.fromstring(get_content(qfile, refresh, url))

        self.publisher = get_encoded_text(self.xml, 'entry/dc:publisher')
        self.title = get_encoded_text(self.xml, 'entry/dc:title')
        self.aggregationType = get_encoded_text(self.xml,
                                                'entry/prism:aggregationType')
        self.prism_url = get_encoded_text(self.xml, 'entry/prism:url')

        # Impact factors
        SNIP = get_encoded_text(self.xml, 'entry/SNIPList/SNIP')
        SNIP_year = self.xml.find('entry/SNIPList/SNIP', ns)
        if SNIP_year is not None:
            SNIP_year = SNIP_year.get('year')
        else:
            SNIP_year = -1

        IPP = get_encoded_text(self.xml, 'entry/IPPList/IPP')
        IPP_year = self.xml.find('entry/IPPList/IPP', ns)
        if IPP_year is not None:
            IPP_year = IPP_year.get('year')
        else:
            IPP_year = -1

        SJR = get_encoded_text(self.xml, 'entry/SJRList/SJR')
        SJR_year = self.xml.find('entry/SJRList/SJR', ns)
        if SJR_year is not None:
            SJR_year = SJR_year.get('year')
        else:
            SJR_year = -1
        if SNIP:
            self.SNIP = float(SNIP)
            self.SNIP_year = int(SNIP_year)
        else:
            self.SNIP = None
            self.SNIP_year = None

        if IPP:
            self.IPP = float(IPP)
            self.IPP_year = int(IPP_year)
        else:
            self.IPP = None
            self.IPP_year = None

        if SJR:
            self.SJR = float(SJR)
            self.SJR_year = int(SJR_year)
        else:
            self.SJR = None
            self.SJR_year = None

        scopus_url = self.xml.find('entry/link[@ref="scopus-source"]')
        if scopus_url is not None:
            self.scopus_url = scopus_url.attrib['href']
        else:
            self.scopus_url = None

        homepage = self.xml.find('entry/link[@ref="homepage"]')
        if homepage is not None:
            self.homepage = homepage.attrib['href']
        else:
            self.homepage = None

    def __str__(self):
        s = """{self.title} {self.scopus_url}
    Homepage: {self.homepage}
    SJR:  {self.SJR} ({self.SJR_year})
    SNIP: {self.SNIP} ({self.SNIP_year})
    IPP:  {self.IPP} ({self.IPP_year})
""".format(self=self)
        return s

    @property
    def org(self):
        """Return an org-formatted string for a Journal."""
        s = """[[{self.scopus_url}][{self.title}]] [[{self.homepage}][homepage]]
| SJR | SNIP | IPP |
| {self.SJR} | {self.SNIP} | {self.IPP} |""".format(self=self)
        return s



from scopus.tests.test_CitationOverview import *
from scopus.tests.test_ScopusAbstract import *
from scopus.tests.test_ScopusAffiliation import *
from scopus.tests.test_ScopusAuthor import *
from scopus.tests.test_ScopusSearch import *
from scopus.tests.test_AuthorSearch import *



import os
import xml.etree.ElementTree as ET

from scopus.utils import get_content, get_encoded_text

SCOPUS_AFFILIATION_DIR = os.path.expanduser('~/.scopus/affiliation')

if not os.path.exists(SCOPUS_AFFILIATION_DIR):
    os.makedirs(SCOPUS_AFFILIATION_DIR)


class ScopusAffiliation:
    @property
    def affiliation_id(self):
        """The Scopus ID of the affiliation."""
        return self._aff_id

    @property
    def date_created(self):
        """Date the Scopus record was created."""
        return self._date_created

    @property
    def nauthors(self):
        """Number of authors in the affiliation."""
        return self._nauthors

    @property
    def ndocuments(self):
        """Number of documents for the affiliation."""
        return self._ndocuments

    @property
    def url(self):
        """URL to the affiliation's profile page."""
        return self._url

    @property
    def api_url(self):
        """URL to the affiliation's API page."""
        return self._api_url

    @property
    def org_type(self):
        """Type of the affiliation."""
        return self._org_type

    @property
    def org_domain(self):
        """Internet domain of the affiliation."""
        return self._org_domain

    @property
    def org_url(self):
        """Website of the affiliation."""
        return self._org_url

    @property
    def name(self):
        """The name of the affiliation."""
        return self._name

    @property
    def address(self):
        """The address of the affiliation."""
        return self._address

    @property
    def city(self):
        """The city of the affiliation."""
        return self._city

    @property
    def country(self):
        """The country of the affiliation."""
        return self._country

    def __init__(self, aff_id, refresh=False):
        """Class to represent an Affiliation in Scopus.

        Parameters
        ----------
        aff_id : str or int
            The Scopus Affiliation ID.

        refresh : bool (optional, default=False)
            Whether to refresh the cached file if it exists or not.

        Notes
        -----
        The files are cached in ~/.scopus/affiliation/{aff_id}.
        """

        qfile = os.path.join(SCOPUS_AFFILIATION_DIR, str(aff_id))
        url = ('https://api.elsevier.com/content/affiliation/'
               'affiliation_id/{}'.format(aff_id))

        xml = ET.fromstring(get_content(qfile, url=url, refresh=refresh))

        # coredata
        self._url = xml.find('coredata/link[@rel="scopus-affiliation"]')
        _aff_id = get_encoded_text(xml, 'coredata/dc:identifier')
        self._aff_id = _aff_id.split(":")[-1]
        if self._url is not None:
            self._url = self.url.get('href')
        self._api_url = get_encoded_text(xml, 'coredata/prism:url')
        self._nauthors = get_encoded_text(xml, 'coredata/author-count')
        self._ndocuments = get_encoded_text(xml, 'coredata/document-count')

        self._name = get_encoded_text(xml, 'affiliation-name')
        self._address = get_encoded_text(xml, 'address')
        self._city = get_encoded_text(xml, 'city')
        self._country = get_encoded_text(xml, 'country')

        # institution-profile
        date_created = xml.find('institution-profile/date-created')
        if date_created is not None:
            self._date_created = (int(date_created.attrib['year']),
                                  int(date_created.attrib['month']),
                                  int(date_created.attrib['day']))
        else:
            self._date_created = (None, None, None)
        self._org_type = get_encoded_text(xml, 'institution-profile/org-type')
        self._org_domain = get_encoded_text(xml, 'institution-profile/org-domain')
        self._org_url = get_encoded_text(xml, 'institution-profile/org-URL')

    def __str__(self):
        s = '''{self.name} ({self.nauthors} authors, {self.ndocuments} documents)
    {self.address}
    {self.city}, {self.country}
    {self.url}'''.format(self=self)
        return s



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for `ScopusSearch` module."""

from nose.tools import assert_equal

import scopus


s = scopus.ScopusSearch('AU-ID(24320488600)', refresh=True)


def test_EIDS():
    assert_equal(s.EIDS, ['2-s2.0-26444452434'])


def test_org_summary():
    expected = '1. [[https://www.scopus.com/inward/record.uri?partnerID=\
HzOxMe3b&scp=26444452434&origin=inward][2-s2.0-26444452434]]  Mario Draghi, \
Economists as policymakers: A round-table discussion. Introduction, European \
Economic Review, 36(2-3), p. 307-309, (1992). https://doi.org/None, \
https://www.scopus.com/inward/record.uri?partnerID=HzOxMe3b&scp=26444452434&\
origin=inward, cited 1 times (Scopus).\n  Affiliations:\n   \n'
    assert_equal(s.org_summary, expected)
